﻿; (function (window, undefined) {

    var App = window.App || {};

    var _callbacks = { };

    /**
     * @private
     * Will convert the specified object into a string value.
     * @param  {Object} obj The object that should be converted into a string value.
     * @return {String}     The string representation of the specified object.
     */
    function _objToUrlString(obj) {

        if (!obj) {
            return '';
        }

        var str = '';
        for (var p in obj) {
            if (obj.hasOwnProperty(p)) {

                var val = obj[p];
                if (typeof val === 'object') {
                    val = JSON.stringify(val);
                }

                str += p + '=' + encodeURIComponent(val) + '&';
            }
        }
        return str.slice(0, -1);
    };

    /**
    * @private
    * Will generate a new GUID.
    * @return {String} The generated GUID.
    */
    function _generateGuid() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }

    function _httpRequest(url, callback) {

        window.XMLHttpRequest = window.XMLHttpRequest || new ActiveXObject("Microsoft.XMLHTTP");

        var xhr = new window.XMLHttpRequest();
        xhr.onreadystatechange = function () {

            if (xhr.readyState == 4) {

                if (!callback) {
                    return;
                }

                if (xhr.responseText !== '') {
                    callback(xhr.responseText);
                }
            }
        };

        try {

            if (callback) {
                var identifier = _generateGuid();
                if (identifier) {
                    _callbacks[identifier] = callback;

                    if (url.indexOf('?') >= 0) {
                        url = url + '&callbackIdentifier=' + identifier;
                    }
                    else {
                        url = url + '?callbackIdentifier=' + identifier;
                    }
                }
            }

            xhr.open("GET", App.BaseUrl + url, true);
            xhr.send();

        }
        catch (e) {

            if (e.name && e.name === 'NETWORK_ERR' && url.indexOf('appmachine/') !== -1) {
                return;
            }

            //App.logError(e.message, 'AppMachine Core', '84', null);
            callback(500);
        }
    }

    /**
     * @private
     * Will replace the core.js execute method and will create a bridge between the JavaScript client running inside the IFrame and the Mobile Client.
     * @param  {String}   type     The name of the method that should be executed.
     * @param  {Object}   args     data send to Mobile Client, object needs to be "clone-able"
     * @param  {Function} callback The method that should be invoked once the method has been executed.
     * @param  {Boolean}  isError  are we logging an error?
     */
    function _execute(type, args, callback, isError) {

        var url = type;

        if (typeof (args) === 'string' && args !== '') {
            url = url + '?' + args;
        }
        else if (typeof (args) === 'object' && args !== null) {
            url = url + '?' + _objToUrlString(args);
        }

        return _httpRequest(url, callback);
    }

    /**
     * @private
     * Will initialize the Mobile Client extensions to core.js.
     */
    function _init() {

        //Register execute
        App.Core.registerMethodReplacement('execute', _execute);
        App.Core.callback = function (identifier, args) {

            if (_callbacks.hasOwnProperty(identifier)) {

                var callback = _callbacks[identifier];
                if (callback) {
                    callback(args);
                }

                delete _callbacks[identifier];
            }

        };
    }

    //<!-- inject[BaseUrl] -->

    _init();
})(window);